function [ecgMatrix, heartBeatDuration, meanECG] = ecg_image_processing_Lb(ECG,fs,Lb) 
% In this script, QRS peaks are aligned, it returns the 2D matrix,
% the mean of the original signal, the length of each hearthbeats and the
% frequence of the sampling. These are all info necessay to the image
% compression and reconstruction process

%% Data preprocessing

meanECG = mean(ECG);
ECG = ECG - mean(ECG);

%% Peak detection

[~, rIndex, ~ ]=pan_tompkin_mute(ECG, fs, 0);

%% Initialize

sectionAligned = [];
sectionGraph = [];
ecgMatrix = [];
row = [];
heartBeatDuration = []; % Duration of each heartbeats

%% Section division
sectionMidRR = [];

sectionMidRR(1) = 1;
for i = 1:length(rIndex) - 1
	sectionMidRR(i+1) = floor(.5 * (rIndex(i) + rIndex(i+1)));
end
sectionMidRR(end + 1) = length(ECG);

% Get the maximum possible length of the R-R distance
maxLengthRR = 0;
for i = 1:length(sectionMidRR) - 1
	n = sectionMidRR(i+1) - sectionMidRR(i);
	if n > maxLengthRR
		maxLengthRR = n;
	end
end

% Get the minimum possible length of the R-R distance
minLengthRR = maxLengthRR;
for i = 1:length(sectionMidRR) - 1
	n = sectionMidRR(i+1) - sectionMidRR(i);
	if n < minLengthRR
		minLengthRR = n;
	end
end

sectionAligned = [];
sectionAligned(1) = 1;
for i = 2:length(rIndex)
	ind = floor(rIndex(i) - (minLengthRR / 2));
	dist = floor(rIndex(i) - (minLengthRR / 2)) - sectionAligned(end);
	
	if dist <= Lb
		sectionAligned = [sectionAligned,  ind];
	else
		q = floor(dist / Lb);
		r = mod(dist, Lb);
		for i = (1:q)
			sectionAligned = [sectionAligned, ind - dist + Lb*i];
		end
		if r ~= 0
			sectionAligned = [sectionAligned, ind];
		end
	end	
end
sectionAligned(end + 1) = length(ECG);

%% Plot the resulting division

% figure
% plot(ECG)
%   
% for i = 1:length(sectionAligned)
% 	sectionGraph(i) = xline(sectionAligned(i), '--r');
% end

%% Put it in a matrix

for i = 1:length(sectionAligned) - 1
	heartBeatDuration(i) = sectionAligned(i+1) - sectionAligned(i);
end
heartBeatDuration(end) = heartBeatDuration(end) + 1;

% Get the maximum section size
maxLengthAligned = 0;
for i = 1:length(sectionAligned) - 1
	n = sectionAligned(i+1) - sectionAligned(i);
	if n > maxLengthAligned
		maxLengthAligned = n;
	end
end

% Put each section in a matrix and fill the rest with 0
for i = 1:length(sectionAligned) - 1
	row = [transpose(ECG(sectionAligned(i):sectionAligned(i+1))), zeros(1, maxLengthAligned - (sectionAligned(i+1) - sectionAligned(i)))];
	ecgMatrix = [ecgMatrix; row];
end

end
